package lifeline.model;

import java.util.ArrayList;

import lifeline.connection.Connection;
import lifeline.connection.DBHandle;

import org.apache.http.HttpResponse;
import org.apache.http.message.BasicNameValuePair;

import android.content.ContentValues;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;

// This class represents a Lifeline (a group)
public class Group extends Model {
	public String name;
	public String desc;
	public String website;
	public static String tableName = "groups";

	public Group() {
		super();
		globalSaved = false;
	}

	public Group(String name, String description, String website) {
		this();
		this.name = name;
		this.website = website;
		desc = description;
		save();
	}

	@Override
	protected ContentValues toContentValues() {
		ContentValues v = new ContentValues();
		v.put("name", name);
		v.put("desc", desc);
		v.put("website", website);
		return v;
	}

	@Override
	protected ArrayList<BasicNameValuePair> toParams() {
		if (website == null)
			website = "";
		if (desc == null)
			desc = "";
		ArrayList<BasicNameValuePair> params = new ArrayList<BasicNameValuePair>();
		params.add(new BasicNameValuePair("name",name));
		params.add(new BasicNameValuePair("desc",desc));
		params.add(new BasicNameValuePair("website",website));
		return params;
	}

	//returns all members of this group
	public ArrayList<User> getMembers() {
		String query = "SELECT u.* FROM users u " +
		"INNER JOIN memberships m ON u._id = m.user_id " +
		"WHERE m.group_id = ?";
		Cursor cur = db.rawQuery(query, new String[]{String.valueOf(_id)});
		ArrayList<User> list = User.fromCursor(cur);
		cur.close();
		return list;
	}

	public ArrayList<Message> getMessages() {
		String query = "SELECT m.* FROM messages m " +
		"WHERE m.group_id = ?";
		Cursor cur = db.rawQuery(query, new String[]{String.valueOf(_id)});
		ArrayList<Message> list = Message.fromCursor(cur);
		cur.close();
		return list;
	}

	public boolean addMessage(String text, User u) {
		Message m = new Message(u, this, text);
		return m.save();
	}

	public boolean addMember(User u) {
		if (isMember(u))
			return false;
		Membership m = new Membership(u, this);
		return m.save();
	}
	
	public boolean isMember(User u) {
		ArrayList<User> users = getMembers();
		for (User user : users) {
			if (user.id() == u.id())
				return true;
		}
		return false;
	}
	
	//given a cursor, returns a list of Groups it contains
	public static ArrayList<Group> fromCursor(Cursor cur) {
		ArrayList<Group> list = new ArrayList<Group>();
		if (cur.moveToFirst()) {
            do {
                // Get the field values
            	Group g = new Group();
            	g._id = Integer.valueOf(cur.getString(cur.getColumnIndex("_id")));
                g.name = cur.getString(cur.getColumnIndex("name"));
                g.desc = cur.getString(cur.getColumnIndex("desc"));
                g.website = cur.getString(cur.getColumnIndex("website"));
                g.globalSaved = true;
                list.add(g);
            } while (cur.moveToNext());
        }
		return list;
	}
	
	//returns all groups saved on the local DB
	public static ArrayList<Group> all() {
		SQLiteDatabase db = DBHandle.getConnection();
		Cursor cur = db.query(tableName, null, null, null, null, null, null);
		ArrayList<Group> list = fromCursor(cur);
		cur.close();
		return list;
	}
	
	//retrieves a group, by ID, from the server
	public static boolean fromServer(String id) {
		HttpResponse resp = Connection.getRequest("/" + tableName + "/" + id);
		if (resp == null || resp.getStatusLine().getStatusCode() != 200)
			return false;
		Group g = new Group();
		g._id = Integer.parseInt(resp.getHeaders("Id")[0].getValue());
		g.name = resp.getHeaders("Name")[0].getValue();
        g.desc = resp.getHeaders("Desc")[0].getValue();
        g.website = resp.getHeaders("Website")[0].getValue();
        g.globalSaved = true;
        
        return g.saveLocally();
	}
	
	//checks the local DB to see if an object with this ID exists
	public static boolean exists(String id) {
		SQLiteDatabase db = DBHandle.getConnection();
		String query = "SELECT * FROM " + tableName + " WHERE _id = ?";
		Cursor cur = db.rawQuery(query, new String[]{ id });
		boolean exists = (cur.getCount() == 1);
		cur.close();
		return exists;
	}
	
	//find an object in the local DB by ID
	public static Group find(String id) {
		SQLiteDatabase db = DBHandle.getConnection();
		String query = "SELECT * FROM " + tableName + " WHERE _id = ?";
		Cursor cur = db.rawQuery(query, new String[]{ id });
		Group found = fromCursor(cur).get(0);
		cur.close();
		return found;
	}
	
	public static ArrayList<Group> findByName(String name) {
		SQLiteDatabase db = DBHandle.getConnection();
		String query = "SELECT * FROM " + tableName + " WHERE name = ?";
		Cursor cur = db.rawQuery(query, new String[]{ name });
		ArrayList<Group> found = fromCursor(cur);
		cur.close();
		return found;
	}
	
	public boolean saveLocally() {
		ContentValues v = toContentValues();
		v.put("_id", _id);
		long b = db.insert(tableName, null, v);
		if (b != _id)
			return false;
		return true;
	}

	@Override
	public String getTableName() {
		return tableName;
	}
}
