package lifeline.service;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;

import lifeline.connection.Connection;
import lifeline.connection.DBHandle;
import lifeline.model.Account;
import lifeline.model.Group;
import lifeline.model.Membership;
import lifeline.model.Message;
import lifeline.model.User;

import org.apache.http.HttpResponse;

import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.util.Log;

//Downloads information from the server to update the local DB
public class Updater implements Runnable {
	private final String TAG = "Updater";
//	private int ONE_MINUTE = 60000;
	private int FIVE_MINUTES = 300000;
	private boolean stopped;

	@Override
	public void run() {
		while(true) {
			//if stop has been called, exit thread
			synchronized (TAG) {
				if (stopped)
					return;
			}
			
			//run the updates
			update();
			
			//wait some time before doing it again
			try {
				Thread.sleep(FIVE_MINUTES);
			} catch (InterruptedException e) {
				e.printStackTrace();

			}
		}
	}

	public void stop() {
		synchronized (TAG) {
			stopped = true;
		}
	}

	public void update() {
		updateGroupsList();
		updateUsersList();

		if (Account.authenticated()) {
			ArrayList<Group> groups = Account.currUser().getLifelines();
			updateUserMemberships(Account.currUser());
			for (Group g : groups) {
				updateGroupMembers(g);
				updateGroupMessages(g);
			}
		}
	}

	public void updateGroupsList() {
		//request a list of groups from server
		try {
			HttpResponse resp = Connection.getRequest("/groups");
			if (resp == null) {
				Log.e(TAG, "updateGroupList error - NULL resp");
				return;
			} else if (resp.getStatusLine().getStatusCode() != 200) {
				Log.e(TAG, "updateGroupList error - not 200");
				return;
			}
			InputStream is = resp.getEntity().getContent();
			BufferedReader br = new BufferedReader(new InputStreamReader(is));
			String line = null;

			//request all groups we don't already have
			while (null != (line = br.readLine())) {
				String[] ids = line.split(" ");
				for (String id : ids) {
					if (!Group.exists(id))
						Group.fromServer(id);
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public void updateUsersList() {
		//request a list of users of a group from the server
		try {
			HttpResponse resp = Connection.getRequest("/users");
			if (resp == null) {
				Log.e(TAG, "updateUsersList error - null resp");
				return;
			} else if (resp.getStatusLine().getStatusCode() != 200) {
				Log.e(TAG, "updateUsersList error - not 200");
				return;
			}
			InputStream is = resp.getEntity().getContent();
			BufferedReader br = new BufferedReader(new InputStreamReader(is));
			String line = null;

			//request all users we don't already have
			while (null != (line = br.readLine())) {
				String[] ids = line.split(" ");
				for (String id : ids) {
					if (!User.exists(id))
						User.fromServer(id);
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public void updateGroupMembers(Group g) {
		//request a list of users of a group from the server
		try {
			String path = "/groups/" + g.id() + "/memberships";
			HttpResponse resp = Connection.getRequest(path);
			if (resp == null) {
				Log.e(TAG, "updateGroupUsers error - resp null");
				return;
			}else if (resp.getStatusLine().getStatusCode() != 200) {
				Log.e(TAG, "updateGroupUsers error - not 200");
				return;
			}	
			InputStream is = resp.getEntity().getContent();
			BufferedReader br = new BufferedReader(new InputStreamReader(is));
			String line = null;

			//request all memberships we don't already have
			while (null != (line = br.readLine())) {
				String[] ids = line.split(" ");
				for (String id : ids) {
					if (!Membership.exists(id))
						Membership.fromServer(id);
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	//update the group the user is in
	public void updateUserMemberships(User u) {
		try {
			String path = "/users/" + u.id() + "/memberships";
			HttpResponse resp = Connection.getRequest(path);
			if (resp == null) {
				Log.e(TAG, "updateUserMemberships error - resp null");
				return;
			}else if (resp.getStatusLine().getStatusCode() != 200) {
				Log.e(TAG, "updateUserMemberships error - not 200");
				return;
			}	
			InputStream is = resp.getEntity().getContent();
			BufferedReader br = new BufferedReader(new InputStreamReader(is));
			String line = null;

			//request all memberships we don't already have
			while (null != (line = br.readLine())) {
				String[] ids = line.split(" ");
				for (String id : ids) {
					if (!Membership.exists(id))
						Membership.fromServer(id);
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public void updateGroupMessages(Group g) {
		//find the last message in the DB that isn't from us
		String query = "SELECT MAX(_id) as _id FROM messages " +
		"WHERE group_id = ? AND user_id != ?";
		int gid = g.id();
		String sgid = String.valueOf(gid);
		int uid = Account.currId();
		String suid = String.valueOf(uid);
		SQLiteDatabase db = DBHandle.getConnection();
		Cursor cur = db.rawQuery(query, new String[]{ sgid, suid });

		int lastMsg = 0;
		if (cur.moveToFirst()) {
			lastMsg = cur.getInt(0); //col 0 is _id, since it's the only one SELECTed
		}
		cur.close();
		
		//request a list of messages newer than lastMsg
		try {
			HttpResponse resp = Connection.getRequest(
					"/groups/" + sgid + "/messages/" + lastMsg);
			if (resp == null || resp.getStatusLine().getStatusCode() != 200)
				Log.w(TAG, "updateGroupMessages error");
			InputStream is = resp.getEntity().getContent();
			BufferedReader br = new BufferedReader(new InputStreamReader(is));
			String line = null;

			//request all messages we don't already have
			while (null != (line = br.readLine())) {
				String[] ids = line.split(" ");
				for (String id : ids) {
					if (!Message.exists(id))
						Message.fromServer(id);
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public void clearLocalDB() {
		SQLiteDatabase db = DBHandle.getConnection();
		db.delete(User.tableName, null, null);
		db.delete(Group.tableName, null, null);
		db.delete(Membership.tableName, null, null);
		db.delete(Message.tableName, null, null);
		update();
	}
}
